==> fetch.php?media=download_software:mae3xx:mae3xx_rootfs_bionic_v4_3_0.tar.xz: usr/local/lib/python3.6/dist-packages/kym11lib.py <==
#! /usr/bin/env python3
# KDDI LTE Module KYM11 manipulation library
#

import os
import re
import time
import syslog
import select
import serial


OTA_results = {
    "NO SERVICED": (True, "Already activated.", True),
    "NO CARRIER": (True, "No carrier.", False),
    "A99": (True, "Call failed.", False),
    "OTASP1": (False, "OTA progress: 1"),
    "OTASP2": (False, "OTA progress: 2"),
    "OTASP3": (False, "OTA progress: 3"),
    "OTASP4": (False, "OTA progress: 4"),
    "A7": (True, "Commit Response message not received.", False),
    "OTASPOK": (True, "OTASP completed.", True),
}


def log(msg, console=False):
    syslog.syslog(syslog.LOG_INFO, msg)
    if console == True:
        print(msg)


def find_KYM():
    dev = None

    if os.path.exists("/tmp/.kym11_found"):
        dev = "KYM11"
    elif os.path.exists("/tmp/.kym12_found"):
        dev = "KYM12"

    return dev


class LCK(object):
    def __init__(self, port):
        self._lckfilename = "/var/lock/LCK..%s" % port
        mypid = os.getpid()
        stale = False

        if os.path.isfile(self._lckfilename):
            try:
                pid = int(open(self._lckfilename, "r").read())
                os.kill(pid, 0)
            except (ValueError, OSError):
                self._locked = False

        open(self._lckfilename, "w").write("%10d\n" % mypid)
        self._locked = True

    def __del__(self):
        try:
            os.unlink(self._lckfilename)
        except:
            pass

    def is_locked(self):
        return self._locked


class ACM(object):
    def __init__(self, port, timeout=0.1):
        self._lck = LCK(port)
        self._dev = "/dev/%s" % port
        self._timeout = timeout
        self.open()

    def with_open(func):
        def wrapper(self, *args):
            try:
                self._s.timeout = self._timeout
            except:
                msg = "kym11lib: KYM11 disconnected, re-open."
                log(msg)
                self.open()

                if not self._s:
                    raise OSError

            return func(self, *args)

        return wrapper

    def open(self):
        self._s = None

        for retry in range(20):
            try:
                self._s = serial.Serial(self._dev, timeout=self._timeout)
                break
            except:
                time.sleep(1)

    def set_timeout(self, timeout):
        self._timeout = timeout

    @with_open
    def communicate(self, atcmd=""):
        res = []

        if self._lck.is_locked() == False:
            return res

        if atcmd:
            self._s.write(("%s\r" % atcmd).encode())

        self._s.timeout = self._timeout
        while True:
            try:
                lines = self._s.readline().decode()
                if not lines:
                    break

                for _line in lines.strip().split("\r"):
                    line = _line.strip()
                    if line:
                        res.append(line)

                self._s.timeout = 0.2
            except serial.SerialException as e:
                msg = "kym11lib: KYM11 disconnected."
                log(msg)
                self._s.close()
                self._s = None
                break
            except:
                break

        return res


class WDM(object):
    def __init__(self, port, timeout=0.5):
        self._lck = LCK(port)
        self._dev = "/dev/%s" % port
        self._timeout = timeout
        self.open()

    def __del__(self):
        if self._f:
            os.close(self._f)

    def with_open(func, *args):
        def wrapper(self, *args):
            if self._f == None:
                self.open()
            return func(self, *args)

        return wrapper

    def open(self):
        self._f = os.open(self._dev, os.O_RDWR)

    def set_timeout(self, timeout):
        self._timeout = timeout

    @with_open
    def communicate(self, atcmd=""):
        res = []

        if atcmd:
            try:
                buf = "%s\r\n" % atcmd
                os.write(self._f, buf.encode())
            except OSError:
                self._f = None
                return res

        timeout = self._timeout

        lines = []
        while True:
            _rfds, _, _ = select.select([self._f], [], [], timeout)

            if not _rfds:
                break

            _line = os.read(self._f, 1024)
            line = _line.decode()

            lines.append(line)
            timeout = 0.1

        _buf = "".join(lines)
        buf = _buf.strip().replace("\n", "").split("\r")

        for entry in buf:
            if entry:
                res.append(entry)

        return res


class KYM11(object):
    def __init__(self):
        dev = find_KYM()

        if dev == None:
            raise SystemError("KYM1x not found")

        self._acm = ACM("tty{}".format(dev))
        self._wdm = WDM("cdc-wdm0", 0.3)
        self._device = dev
        self._wdm_checked = False

    def via_wdm(func):
        def wrapper(self, *args):
            if self._wdm_checked or self.is_current_mode_AT() or self.set_AT_mode():
                self._wdm_checked = True
                return func(self, *args)
            else:
                return func(self, *args)

        return wrapper

    def is_current_mode_AT(self):
        res = self._acm.communicate("AT$99?")
        if not res or len(res) < 2 or res[-1] != "OK" or res[-2] != "3":
            return False
        else:
            return True

    def set_AT_mode(self):
        res = self._acm.communicate("AT$99=3")
        if not res or len(res) < 1 or res[-1] != "OK":
            return False
        else:
            return True

    def get_device(self):
        return self._device

    def get_datetime(self):
        res = self._acm.communicate("AT$31?")

        if type(res) == list and len(res) >= 2 and res[-1] == "OK":
            return res[-2]
        else:
            return "2000/01/01 00:00:00"

    def get_pktstat(self):
        pktstats = {1: "Detached", 3: "Idle", 4: "Connected", -1: "Unknown"}

        res = self._acm.communicate("AT$36?")

        if type(res) == list and len(res) >= 2 and res[-1] == "OK":
            code = int(res[-2])
        else:
            code = -1

        return code, pktstats[code]

    def get_tempprotect(self):
        stats = {0: "Normal", 1: "Protected", -1: "Unknown"}

        res = self._acm.communicate("AT$39?")

        if type(res) == list and len(res) >= 2 and res[-1] == "OK":
            code = int(res[-2])
        else:
            print("get_tempprotect: res={}".format(res))
            code = -1

        return code, stats.get(code, "Unknown")

    def get_version(self):
        res = self._acm.communicate("AT+GMR")

        if type(res) == list and len(res) >= 2 and res[-1] == "OK":
            version = res[-2].split()[1]
        else:
            version = "Unknown"

        return version

    def get_simstat(self):
        stats = {
            0: "Not registered",
            1: "Registered, Ready",
            2: "Registered, Not ready",
        }

        res = self._acm.communicate("AT$20?")

        if type(res) == list and len(res) >= 2 and res[-1] == "OK":
            code = int(res[-2])
        else:
            code = -1

        return code, stats.get(code, "Unknown")

    def get_telno(self):
        res = self._acm.communicate("AT$21?")

        if type(res) == list and len(res) >= 2 and res[-1] == "OK":
            tel_no = res[-2]
        else:
            tel_no = "Unknown"

        return tel_no

    def get_sim(self):
        res = self._acm.communicate("AT$18?")

        if type(res) == list and len(res) >= 2 and res[-1] == "OK":
            current = int(res[-2].split(",")[1])
        else:
            current = -1

        return current

    def select_sim(self, slot):
        current = self.get_sim()

        if slot == True:
            sel = 2
        elif slot == False:
            sel = 1
        else:
            return False

        if sel == current:
            return True

        res = self._acm.communicate("AT$18=%d" % sel)

        if type(res) == list and len(res) >= 1 and res[-1] == "OK":
            return True
        else:
            return False

    def get_imei(self):
        res = self._acm.communicate("AT$10?")

        if type(res) == list and len(res) >= 2 and res[-1] == "OK":
            imei = res[-2]
        else:
            imei = "Unknown"

        return imei

    def get_iccid(self):
        res = self._acm.communicate("AT$19?")

        if type(res) == list and len(res) >= 2 and res[-1] == "OK":
            iccid = res[-2]
        else:
            iccid = "Unknown"

        return iccid

    def get_level(self):
        res = self._acm.communicate("AT$30=0")

        if type(res) == list and len(res) >= 2 and res[-1] == "OK":
            level = res[-2]
        else:
            level = -1

        return level

    def get_error_report(self):
        res = []
        first = True

        while True:
            if first:
                _res = self._acm.communicate("AT$59=1,10")
                first = False
            else:
                _res = self._acm.communicate("AT$59=0")

            if type(_res) == list and len(_res) >= 3 and _res[-1] == "OK":
                _res = _res[1:-1]
                res.extend(_res)

                if len(_res) < 10:
                    break
            else:
                break

        return res

    def OTA(self, timeout=300):
        self._acm.set_timeout(30)
        res = self._acm.communicate("AT@30")

        if type(res) == list and len(res) > 1:
            if res[-1] == "OK":
                log("OTASP started.", True)
            else:
                info = OTA_results.get(res[-1])
                if info:
                    log(info[1], True)
                    if info[0] == True:
                        return info[2]
                else:
                    log("Unhandled result '%s' received." % res[-1])
                    return False
        else:
            log("OTASP start failed.", True)
            return False

        starttime = time.time()
        while True:
            res = self._acm.communicate()
            now = time.time()

            if not res or type(res) != list or len(res) < 1:
                if now - starttime < timeout:
                    continue
                else:
                    log("OTA timeouted.", True)
                    return False

            for msg in res:
                info = OTA_results.get(msg)
                if not info:
                    log("Undefined result received, '%s'" % msg, True)
                    continue
                log(info[1], True)
                if info[0] == True:
                    # finish
                    return info[2]


# sample
def main():
    kym = KYM11()

    date_time = kym.get_datetime()
    pkt_stat = kym.get_pktstat()
    temp_stat = kym.get_tempprotect()
    mod_version = kym.get_version()
    simstat = kym.get_simstat()
    tel_no = kym.get_telno()
    iccid = kym.get_iccid()
    imei = kym.get_imei()
    level = kym.get_level()
    err_report = kym.get_error_report()

    print("== {} status ==".format(kym.get_device()))
    print(" Date/Time:       %s" % date_time)
    print(" Packet Status:   %s" % pkt_stat[1])
    print(" Temp Protection: %s" % temp_stat[1])
    print(" Module version:  %s" % mod_version)
    print(" SIM status:      %s" % simstat[1])
    print(" Telephone No:    %s" % tel_no)
    print(" IMEI:            %s" % imei)
    print(" ICCID:           %s" % iccid)
    print(" Antenna Level:   %s" % level)

    print("----- Error Report -----")
    print("\n".join(err_report))
    print("------------------------")


if __name__ == "__main__":
    main()

==> fetch.php?media=download_software:mae3xx:mae3xx_rootfs_bionic_v4_5_0rc5.tar.xz: usr/local/lib/python3.6/dist-packages/kym11lib.py <==
#! /usr/bin/env python3
# KDDI LTE Module KYM11 manipulation library
#

import os
import re
import time
import syslog
import select
import serial


OTA_results = {
    "NO SERVICED": (True, "Already activated.", True),
    "NO CARRIER": (True, "No carrier.", False),
    "A99": (True, "Call failed.", False),
    "OTASP1": (False, "OTA progress: 1"),
    "OTASP2": (False, "OTA progress: 2"),
    "OTASP3": (False, "OTA progress: 3"),
    "OTASP4": (False, "OTA progress: 4"),
    "A7": (True, "Commit Response message not received.", False),
    "OTASPOK": (True, "OTASP completed.", True),
}


def log(msg, console=False):
    syslog.syslog(syslog.LOG_INFO, msg)
    if console == True:
        print(msg)


def find_KYM():
    dev = None

    if os.path.exists("/tmp/.kym11_found"):
        dev = "KYM11"
    elif os.path.exists("/tmp/.kym12_found"):
        dev = "KYM12"

    return dev


class LCK(object):
    def __init__(self, port):
        self._lckfilename = "/var/lock/LCK..%s" % port
        mypid = os.getpid()
        stale = False

        if os.path.isfile(self._lckfilename):
            try:
                pid = int(open(self._lckfilename, "r").read())
                os.kill(pid, 0)
            except (ValueError, OSError):
                self._locked = False

        open(self._lckfilename, "w").write("%10d\n" % mypid)
        self._locked = True

    def __del__(self):
        try:
            os.unlink(self._lckfilename)
        except:
            pass

    def is_locked(self):
        return self._locked


class ACM(object):
    def __init__(self, port, timeout=0.1):
        self._lck = LCK(port)
        self._dev = "/dev/%s" % port
        self._timeout = timeout
        self.open()

    def with_open(func):
        def wrapper(self, *args):
            try:
                self._s.timeout = self._timeout
            except:
                msg = "kym11lib: KYM11 disconnected, re-open."
                log(msg)
                self.open()

                if not self._s:
                    raise OSError

            return func(self, *args)

        return wrapper

    def open(self):
        self._s = None

        for retry in range(20):
            try:
                self._s = serial.Serial(self._dev, timeout=self._timeout)
                break
            except:
                time.sleep(1)

    def set_timeout(self, timeout):
        self._timeout = timeout

    @with_open
    def communicate(self, atcmd=""):
        res = []

        if self._lck.is_locked() == False:
            return res

        if atcmd:
            self._s.write(("%s\r" % atcmd).encode())

        self._s.timeout = self._timeout
        while True:
            try:
                lines = self._s.readline().decode()
                if not lines:
                    break

                for _line in lines.strip().split("\r"):
                    line = _line.strip()
                    if line:
                        res.append(line)

                self._s.timeout = 0.2
            except serial.SerialException as e:
                msg = "kym11lib: KYM11 disconnected."
                log(msg)
                self._s.close()
                self._s = None
                break
            except:
                break

        return res


class WDM(object):
    def __init__(self, port, timeout=0.5):
        self._lck = LCK(port)
        self._dev = "/dev/%s" % port
        self._timeout = timeout
        self.open()

    def __del__(self):
        if self._f:
            os.close(self._f)

    def with_open(func, *args):
        def wrapper(self, *args):
            if self._f == None:
                self.open()
            return func(self, *args)

        return wrapper

    def open(self):
        self._f = os.open(self._dev, os.O_RDWR)

    def set_timeout(self, timeout):
        self._timeout = timeout

    @with_open
    def communicate(self, atcmd=""):
        res = []

        if atcmd:
            try:
                buf = "%s\r\n" % atcmd
                os.write(self._f, buf.encode())
            except OSError:
                self._f = None
                return res

        timeout = self._timeout

        lines = []
        while True:
            _rfds, _, _ = select.select([self._f], [], [], timeout)

            if not _rfds:
                break

            _line = os.read(self._f, 1024)
            line = _line.decode()

            lines.append(line)
            timeout = 0.1

        _buf = "".join(lines)
        buf = _buf.strip().replace("\n", "").split("\r")

        for entry in buf:
            if entry:
                res.append(entry)

        return res


class KYM11(object):
    def __init__(self):
        dev = find_KYM()

        if dev == None:
            raise SystemError("KYM1x not found")

        self._acm = ACM("tty{}".format(dev))
        self._wdm = WDM("cdc-wdm0", 0.3)
        self._device = dev
        self._wdm_checked = False

    def via_wdm(func):
        def wrapper(self, *args):
            if self._wdm_checked or self.is_current_mode_AT() or self.set_AT_mode():
                self._wdm_checked = True
                return func(self, *args)
            else:
                return func(self, *args)

        return wrapper

    def is_current_mode_AT(self):
        # only ACM
        res = self._acm.communicate("AT$99?")
        if not res or len(res) < 2 or res[-1] != "OK" or res[-2] != "3":
            return False
        else:
            return True

    def set_AT_mode(self):
        # only ACM
        res = self._acm.communicate("AT$99=3")
        if not res or len(res) < 1 or res[-1] != "OK":
            return False
        else:
            return True

    def get_device(self):
        return self._device

    def get_datetime(self, *, use_wdm=False):
        if use_wdm:
            port = self._wdm
        else:
            port = self._acm

        res = port.communicate("AT$31?")

        if type(res) == list and len(res) >= 2 and res[-1] == "OK":
            return res[-2]
        else:
            return "2000/01/01 00:00:00"

    def get_pktstat(self):
        # only WDM
        pktstats = {1: "Detached", 3: "Idle", 4: "Connected", -1: "Unknown"}

        res = self._wdm.communicate("AT$36?")

        if type(res) == list and len(res) >= 2 and res[-1] == "OK":
            code = int(res[-2])
        else:
            code = -1

        return code, pktstats[code]

    def get_tempprotect(self):
        stats = {0: "Normal", 1: "Protected", -1: "Unknown"}

        res = self._acm.communicate("AT$39?")

        if type(res) == list and len(res) >= 2 and res[-1] == "OK":
            code = int(res[-2])
        else:
            print("get_tempprotect: res={}".format(res))
            code = -1

        return code, stats.get(code, "Unknown")

    def get_version(self):
        res = self._acm.communicate("AT+GMR")

        if type(res) == list and len(res) >= 2 and res[-1] == "OK":
            version = res[-2].split()[1]
        else:
            version = "Unknown"

        return version

    def get_simstat(self):
        stats = {
            0: "Not registered",
            1: "Registered, Ready",
            2: "Registered, Not ready",
        }

        res = self._acm.communicate("AT$20?")

        if type(res) == list and len(res) >= 2 and res[-1] == "OK":
            code = int(res[-2])
        else:
            code = -1

        return code, stats.get(code, "Unknown")

    def get_telno(self):
        res = self._acm.communicate("AT$21?")

        if type(res) == list and len(res) >= 2 and res[-1] == "OK":
            tel_no = res[-2]
        else:
            tel_no = "Unknown"

        return tel_no

    def get_sim(self):
        # only ACM
        res = self._acm.communicate("AT$18?")

        if type(res) == list and len(res) >= 2 and res[-1] == "OK":
            current = int(res[-2].split(",")[1])
        else:
            current = -1

        return current

    def select_sim(self, slot):
        # only ACM
        current = self.get_sim()

        if slot == True:
            sel = 2
        elif slot == False:
            sel = 1
        else:
            return False

        if sel == current:
            return True

        res = self._acm.communicate("AT$18=%d" % sel)

        if type(res) == list and len(res) >= 1 and res[-1] == "OK":
            return True
        else:
            return False

    def get_imei(self):
        res = self._acm.communicate("AT$10?")

        if type(res) == list and len(res) >= 2 and res[-1] == "OK":
            imei = res[-2]
        else:
            imei = "Unknown"

        return imei

    def get_iccid(self):
        # only ACM
        res = self._acm.communicate("AT$19?")

        if type(res) == list and len(res) >= 2 and res[-1] == "OK":
            iccid = res[-2]
        else:
            iccid = "Unknown"

        return iccid

    def get_level(self):
        res = self._acm.communicate("AT$30=0")

        if type(res) == list and len(res) >= 2 and res[-1] == "OK":
            level = res[-2]
        else:
            level = -1

        return level

    def get_error_report(self):
        # only WDM
        res = []
        first = True

        while True:
            if first:
                _res = self._wdm.communicate("AT$59=1,10")
                first = False
            else:
                _res = self._wdm.communicate("AT$59=0")

            if type(_res) == list and len(_res) >= 3 and _res[-1] == "OK":
                _res = _res[1:-1]
                res.extend(_res)

                if len(_res) < 10:
                    break
            else:
                break

        return res

    def OTA(self, timeout=300):
        # only ACM
        self._acm.set_timeout(30)
        res = self._acm.communicate("AT@30")

        if type(res) == list and len(res) > 1:
            if res[-1] == "OK":
                log("OTASP started.", True)
            else:
                info = OTA_results.get(res[-1])
                if info:
                    log(info[1], True)
                    if info[0] == True:
                        return info[2]
                else:
                    log("Unhandled result '%s' received." % res[-1])
                    return False
        else:
            log("OTASP start failed.", True)
            return False

        starttime = time.time()
        while True:
            res = self._acm.communicate()
            now = time.time()

            if not res or type(res) != list or len(res) < 1:
                if now - starttime < timeout:
                    continue
                else:
                    log("OTA timeouted.", True)
                    return False

            for msg in res:
                info = OTA_results.get(msg)
                if not info:
                    log("Undefined result received, '%s'" % msg, True)
                    continue
                log(info[1], True)
                if info[0] == True:
                    # finish
                    return info[2]


# sample
def main():
    kym = KYM11()

    date_time = kym.get_datetime()
    pkt_stat = kym.get_pktstat()
    temp_stat = kym.get_tempprotect()
    mod_version = kym.get_version()
    simstat = kym.get_simstat()
    tel_no = kym.get_telno()
    iccid = kym.get_iccid()
    imei = kym.get_imei()
    level = kym.get_level()
    at_mode = kym.is_current_mode_AT()
    err_report = kym.get_error_report()

    print("== {} status ==".format(kym.get_device()))
    print(" Date/Time:       %s" % date_time)
    print(" Packet Status:   %s" % pkt_stat[1])
    print(" Temp Protection: %s" % temp_stat[1])
    print(" Module version:  %s" % mod_version)
    print(" SIM status:      %s" % simstat[1])
    print(" Telephone No:    %s" % tel_no)
    print(" IMEI:            %s" % imei)
    print(" ICCID:           %s" % iccid)
    print(" Antenna Level:   %s" % level)
    print(" Is WDM AT Mode?: %s" % at_mode)

    print("----- Error Report -----")
    print("\n".join(err_report))
    print("------------------------")


if __name__ == "__main__":
    main()
